# EasyPrompt
# by Evan Parker (@evprkr)
# https://github.com/evprkr/EasyPrompt

# Features to add
# - 'get_selection' function for displaying a list of options and taking input to trigger specific callbacks
# - Find a way to specify types accepted as arguments for commands (requires int, str, etc)

# Known bugs

import inspect, shlex

# Main Commander Class
class Commander:
    def __init__(self, prompt=None, newline=True, commands=None, previous_cmd=None):
        if prompt == None: self.prompt = '> '
        else: self.prompt = prompt

        self.newline = newline
        self.commands = []
        self.previous_cmd = previous_cmd

    def get_input(self, prompt=None, no_command=None):
        if prompt is None: prompt = self.prompt

        cmd_input = input(prompt)

        if cmd_input == '!!':
            if self.previous_cmd != None:
                self.parse_command(self.previous_cmd)
            else:
                print("Error: no command history")
                return "Error: no command history"
        elif cmd_input == '':
            print("Error: no command entered")
            return "Error: no command entered"
        else:
            cmd = cmd_input.split()[0]
            for command in self.commands:
                if cmd in command:
                    return self.parse_command(cmd_input)
            else:
                if no_command == None:
                    print(f"Error: command not found: '{cmd}'")
                    if self.newline == True: print('')
                    return "Error: command not found"
                else:
                    callback = no_command
                    callback(cmd_input)

    def parse_command(self, cmd_input):
        self.previous_cmd = cmd_input 
        #input_split = shlex.split(shlex.quote(cmd_input))
        input_split = cmd_input.split()

        for command in self.commands:
            if input_split[0] in command:
                callback = command[1]
                arg_num = len(inspect.getfullargspec(command[1]).args)

                if len(input_split)-1 != arg_num:
                    print(f'Error: expected {arg_num} arguments, got {len(input_split)-1}')
                    if self.newline == True: print('')
                    return "Error: wrong number of arguments"

                if len(input_split) > 1:
                    cmd_args = []
                    for i in range(1, arg_num+1):
                        cmd_args.append(input_split[i])

                    cb_return = callback(*cmd_args)
                    if self.newline == True: print('')
                    return cb_return

                cb_return = callback()
                if self.newline == True: print('')
                return cb_return
        else:
            print(f"Command not found: '{input_split[0]}'")
            if self.newline == True: print('')
            return "Error: command not found"

    def add_command(self, trigger, callback, args=None):
       command = [trigger, callback, args]
       self.commands.append(command)
